function [delay, pos] = CalcDelay(data, sampleTime, trigLvl, points)
% CALCDELAY Searches data for a burst that is longer than MIN_WIDTH
% Returns the start index of this burst or -1 if no burst meeting this
% criteria was found
% delay        Vector containing the phone, rf and tv display
%                   delay for miracast in milliseconds
% pos          Position as an index in the data vector of the
%                   first occurence of the data (for phone, rf, tv)
% data         Vector that contains all data points of the signal
%                   line 1 contains data from the phone photo diode
%                   line 2 contains the modulated rf Data
%                   line 3 contains data from the TV photo diode
% sampleTime   sampling time between each point
% trigLvl      voltage level used by the oscilloscope to trigger
%                   on the diodes
% points       length of the measurement data vector
WINDOW_WIDTH = 10; % Defines how many times larger than the inner burst 
                   % distance between spikes the search window for the rf
                   % burst should be
    
    % Calculate point in time when smartphone display changes the picture
    t_phone = find(data(1,:)>trigLvl,1);
    if isempty(t_phone)        
        error('Source timing not available');
    end;
    
    % Calculate point in time when the TV displays the new picture
    t_tv = find(data(3,:)>trigLvl,1);
    if isempty(t_tv)       
        error('Sink timing not available');
    end;
    
    % Calculate point in time when RF data is in the air
    % 2 possibilities. You have to decide and comment the other one out
    %% Possibility 1: No further toolboxes needed
    % Low Pass Filter the data signal and look for area with many spikes
    % Then search for the first spike in a burst by converting the signal
    % to binary data using a threshold value
    start_idx = t_phone;
    stop_idx = t_tv;
    modDat = filter(ones(1,ceil(points*0.05))/ceil(points*0.05),1,data(2,:));
    % Discard data at the very beginning and end -> better stability
    pulse_range = start_idx:stop_idx;  
    lvl = ((max(modDat(pulse_range))-min(modDat(pulse_range)))/2+min(modDat(pulse_range)));
    % Find first pulse in filtered signal that correspons to a longer burst
    range_end = find(modDat(start_idx:length(modDat))>lvl,1) + start_idx;    
    bin_Dat = (data(2,start_idx:range_end)>max(data(2,start_idx:range_end))/2);
    range_end = range_end - start_idx + 1; % Only range that is of interest
    % Mean distance between spikes in burst area -> determines search area
    m_dist = ceil(mean(diff(find(diff(bin_Dat(floor(0.7*range_end):range_end))))));
    search_range = range_end-m_dist*WINDOW_WIDTH:range_end;
    % now go back in data signal and find first spike in this burst
    while(find(bin_Dat(search_range)>=0.9,1))
        t_rf = find(bin_Dat(search_range)>=0.9,1) + search_range(1) + start_idx - 2;
        search_range = search_range-m_dist*WINDOW_WIDTH;
        if any(search_range<0)
            error('RF timing not available');
        end
    end
    %% Possibility 2: Signal Processing toolbox needed. Performance may be better
     % Convolve RF Data with gauss window --> Low Pass Filter
%     g = gausswin(ceil(points*0.01)); %1 percent of length
%     g = g/(sum(g));
%     modDat = conv(data(2,:),g,'same');
%     % Get the width of all pulses
%     pulse_range = find(modDat>mean(modDat),1,'First'):find(modDat>mean(modDat),1,'Last');
%     % Cut highest pulses off in order to simplify the search
%     lvl = (max(modDat(pulse_range))-mean(modDat(pulse_range)))/2+mean(modDat(pulse_range));
%     modDat(modDat>lvl) = lvl;
%     if t_phone > 0
%         search_range = round(0.8*t_phone):length(modDat);
%     else
%         search_range = 1:length(modDat);
%     end
%     [x,y] = pulsewidth(modDat(search_range),'StateLevels',[min(modDat(pulse_range)) lvl]);
%     % Calc threshold
%     if ~ isempty(x)
%         t_rf = search_range(1)+floor(y(1));
%     else t_rf = -1;
%     end    
    %%
    
    delay = [t_rf-t_phone, t_tv-t_phone];
    delay = delay*sampleTime*1e3;
    pos = [t_phone, t_rf, t_tv];
    pos = pos*sampleTime;