classdef instr_FSx < handle
    % INSTR_FSX Provides essential functions for setting up and controling the instrument
    
       properties
            % Structure for up to 4 channels. Includes: 'enabled', 'vertRange', 'vertOffset', 'vertCoupling'. See VXI pnp drivers -> Configure Channel
            centFreq = 5.192e9;
            instrID = '';   % VISA instrument ID (eg. 'rto-400101')
            connected = 0;  % Intsrument connected (0/1)
       end
       properties (Hidden)
            connection; 
       end
       methods
       % methods, including the constructor are defined in this block
           %Constructor
           function obj = instr_FSx(VxiID)
                obj.instrID = VxiID;
                disp('Loading FSx driver...');
                obj.connection = icdevice('rsspecan.mdd', VxiID);
           end
           
           function obj = Connect(obj)
           %   CONNECT Establish the connection to the device
           try
                disp('Connecting to FSx...');
                connect(obj.connection);
                state = obj.connection.Status;
                if strcmp(state,'open')
                    obj.connected = 1;
                else 
                    obj.connected = 0;
                end
           catch err
                msgbox(err.message);
                obj.connected = 0;
           end
           end
           
           function Disconnect(obj)
               %   DISCONNECT Disconnect from the instrument                        
                disconnect(obj.connection);
                state = obj.connection.Status;
                if strcmp(state,'open')
                    obj.connected = 1;                    
                else 
                    obj.connected = 0;                    
                end   
                delete(obj.connection);
           end
           
          function amplitude = GetAcquisition(obj, traceNo, numPoints)    
          %GETACQUISITION Do one measurement and return waveform data
               %    amplitude = GETACQUISITION(traceNo) returns the waveform data
               %    for the specified traces(s) eg. 1,2,3,4 or [1 2], [1 3 4], ...
            try
              obj.writeRead(['SWEep:POINts ' num2str(numPoints) ';*OPC?']);
              trace = traceNo(1);
              amplitude(length(traceNo),:) = zeros (numPoints, 1);
              [~, amplitude(1,:)] = invoke (obj.connection, 'FetchYTrace', 0, trace, numPoints, amplitude(1,:));
              for i=2:length(traceNo)
                    trace = traceNo(1);
                    [~,amplitude(i,:)] = invoke(obj.connection, 'FetchYTrace', 0, trace, numPoints,amplitude(i,:));
              end
              invoke(obj.connection, 'WriteInstrData','INITiate:CONTinuous ON');
            catch
               obj.getErrorMsg();
            end
          end

          function ConfigureForMiracast(obj, centFreq, numPoints,type)
           %    CONFIGUREFORMIRACAST COnfigures the instrument for the
           %    Miracast measurement
           %    trigLvl         Level to which the trigger will be set
            disp('Configuring FSx...');
            invoke(obj.connection, 'SetSANMode'); % spectrum analyzer mode
            invoke(obj.connection, 'ConfigureReferenceLevel', 0,-10);            
            invoke(obj.connection, 'ConfigureAttenuation', 0,0,0);            
            invoke(obj.connection, 'ConfigureFrequencyCenter', 0,centFreq);            
            invoke(obj.connection, 'ConfigureFrequencyZeroSpan', 0);            
            invoke(obj.connection, 'ConfigureTriggerSource', 0, 1); % external trigger                     
            invoke(obj.connection, 'WriteInstrData',['SWEep:POINts ' num2str(numPoints)]);
            if type == 1 % RTO
                invoke(obj.connection, 'WriteInstrData','Sweep:Time 1');
            else % HMO
                invoke(obj.connection, 'WriteInstrData','Sweep:Time 1.2');
            end
            invoke(obj.connection, 'WriteInstrData','INITiate:CONTinuous OFF');
            obj.writeRead('Sense:Band 20e6; *OPC?');
            obj.writeRead('Sense:Band:VID 20e3; *OPC?');    
            if type == 1 % RTO
                invoke(obj.connection, 'ConfigureTrigger',  0, -0.5, 1); % Pos. val
            else % HMO
                invoke(obj.connection, 'ConfigureTrigger',  0, -0.6, 1); % Pos. val
            end
            invoke(obj.connection, 'ConfigureIFOutput', 1, 2); % Source Vid
          end
                           
           function reset(obj)
               % RESET resets the device
                obj.writeRead('*rst; *OPC?');
                pause(1);
           end
           
           function ret = writeRead(obj,msg)
               % WRITEREAD writes a scpi command to the device and queries the
               % answer
                invoke(obj.connection,'WriteInstrData',msg);
                read = 0;
                Buf = '';
                ret = invoke(obj.connection,'ReadInstrData',256,Buf,read);
           end
          
          function getErrorMsg(obj)
              % GETERRORMSG Query the instrument error state
                invoke(obj.connection,'WriteInstrData','Syst:err?');
                read = 0;
                Buf = '';
                ret = invoke(obj.connection,'ReadInstrData',256,Buf,read);
                disp(['Instrument error status:' ret]);
           end
           
       end       
end 