classdef instr_RTx < handle
    % INSTR_RTX Provides essential functions for setting up and controling the instrument
    
       properties
            acqTime = 20e-6; % Acquisition time in seconds
            sampleRate = 5e9;  % Sample rate in Hertz
            instrID = '';   % VISA instrument ID (eg. 'rto-400101')
            connected = 0;  % Intsrument connected (0/1)
       end
       properties (Hidden)
            connection; 
       end
       methods
       % methods, including the constructor are defined in this block
           %Constructor
           function obj = instr_RTx(VxiID)
                obj.instrID = VxiID;
                disp('Loading RTx driver...');
                obj.connection = icdevice('rsscope.mdd', VxiID);
           end
           
           function obj = Connect(obj)
           %   CONNECT Establish the connection to the device
           try
                disp('Connecting to RTx...');
                connect(obj.connection);
                state = obj.connection.Status;
                if strcmp(state,'open')
                    obj.connected = 1;
                else 
                    obj.connected = 0;
                end
           catch err
                msgbox(err.message);
                obj.connected = 0;
           end
           end
           
           function Disconnect(obj)
               %   DISCONNECT Disconnect from the instrument                        
                disconnect(obj.connection);
                state = obj.connection.Status;
                if strcmp(state,'open')
                    obj.connected = 1;                    
                else 
                    obj.connected = 0;                    
                end   
                delete(obj.connection);
           end
           
           function ConfigureForMiracast(obj, trigLvl, use3Chan)
               %    CONFIGUREFORMIRACAST COnfigures the instrument for the
               %    Miracast measurement
               %    trigLvl         Level to which the trigger will be set
               %    use3Chan        If IF Demod is used, the 3rd channel is
               %                        configured
               disp('Configuring RTx...');
               invoke(obj.connection, 'ConfigureTimeSelectDependency', 1);    
               invoke(obj.connection, 'ConfigureAcquisitionTime', 1); 
               invoke(obj.connection, 'WaveformExportRawValues', 1);
               invoke(obj.connection, 'Abort');
               invoke(obj.connection, 'ConfigureDisplayUpdate', 1);                             
               invoke(obj.connection, 'ConfigureChannel', 1, 1, 0.5, -2.0, 1);
               invoke(obj.connection, 'ConfigureChannel', 2, 1, 0.5, -2.0, 1);
               if use3Chan
                   invoke(obj.connection, 'ConfigureChannel', 3, 1, 0.2, -3.5, 1);                   
               end
               invoke(obj.connection, 'WriteInstrData', 'TIMebase:SCALe 100e-3');               
               invoke(obj.connection, 'WriteInstrData', 'CHANnel:SCALe 50e-3'); 
               % DSP-Filter CH1 100KHz
               invoke(obj.connection, 'WriteInstrData', 'CHANnel:DIGFilter:CUToff 100e3'); 
               % Ch1 Bandwith 20MHz
               invoke(obj.connection, 'ConfigureChannelBandwidth', 1, 3);
               invoke(obj.connection, 'ConfigureTriggerChannelLevel', 1, 1, str2double(trigLvl));
               % set Triggermode Normal
               invoke(obj.connection, 'ConfigureTriggerModifier', 1,0);               
               invoke(obj.connection, 'WriteInstrData', 'TRIGger:HOLDoff:TIME 1.3');
               invoke(obj.connection, 'WriteInstrData', 'TRIGger:OUT:STATe 1');           
               invoke(obj.connection, 'WriteInstrData', 'FORM ASC');
               invoke(obj.connection, 'WriteInstrData', 'EXP:WAV:INCX OFF');             
               invoke(obj.connection,'WriteInstrData','Run');     
           end
                      
            function [wavAcq, actPoints] = GetAcquisition(obj, channelNo, numPoints)    
               % GETACQUISITION Do one measurement and return waveform data
               % wavAcq         Matrix containing all the acquisition data.
               % actPoints      Number of points measured by the device
               % channelNo      Specifies which channels will be measured.
               %                    Multiple channels can be measured in 
               %                    one function call.
               %                    eg. 1 or 2 or 3 or 4 or [1 2], [1 3 4], ...
               % numPoints      Number of acquisition points per channel
               try
                   obj.writeRead(['Acq:Points ' num2str(numPoints) '; *OPC?']);
                   invoke(obj.connection,'InitiateAcquisition',0);
                   channel = channelNo(1);       
                   tmp = zeros(1,numPoints);
                   wavAcq(1,:) = invoke(obj.connection, 'readwaveform', channel,1,numPoints,5000,tmp,0,0,0);
                   for i=2:length(channelNo)
                    channel = channelNo(i);
                    [wavAcq(i,:),actPoi,~,~] = invoke(obj.connection, 'fetchwaveform',  channel,1,numPoints,tmp,0,0,0);                    
                   end
                   obj.writeRead('Run; *OPC?'); 
                   actPoints = length(wavAcq(1,:));
               catch
                   obj.getErrorMsg();
               end
            end
           
           function reset(obj)
               % RESET resets the device
                obj.writeRead('*rst; *OPC?');
                pause(1);
           end
           
           function ret = writeRead(obj,msg)
               % WRITEREAD writes a scpi command to the device and queries the
               % answer
                invoke(obj.connection,'WriteInstrData',msg);
                read = 0;
                Buf = '';
                ret = invoke(obj.connection,'ReadInstrData',256,Buf,read);
           end
           
           function getErrorMsg(obj)
                % GETERRORMSG Query the instrument error state
                invoke(obj.connection,'WriteInstrData','Syst:err?');
                read = 0;
                Buf = '';
                ret = invoke(obj.connection,'ReadInstrData',256,Buf,read);
                disp(['Instrument error status:' ret]);
           end
           
       end       
end 